@@ -34,6 +34,10 @@
 #include "cross.h"
 #include "control.h"
 #include "dos_network2.h"
+#include "menu.h"
+#include "cdrom.h"
+#include "ide.h"
+#include "bios_disk.h"
 
 #define DOS_FILESTART 4
 
@@ -729,6 +733,7 @@ bool DOS_CreateFile(char const * name,Bit16u attributes,Bit16u * entry,bool fcb)
 		if (Files[handle]) Drives[drive]->EmptyCache();
 		return true;
 	} else {
+		if(dos.errorcode==DOSERR_ACCESS_DENIED||dos.errorcode==DOSERR_WRITE_PROTECTED) return false;
 		if(!PathExists(name)) DOS_SetError(DOSERR_PATH_NOT_FOUND); 
 		else DOS_SetError(DOSERR_FILE_NOT_FOUND);
 		return false;
@@ -1885,6 +1890,7 @@ bool DOS_SetFileDate(Bit16u entry, Bit16u ntime, Bit16u ndate)
 	return true;
 }
 
+extern int swapInDisksSpecificDrive;
 void DOS_SetupFiles (void) {
 	/* Setup the File Handles */
 	Files = new DOS_File * [DOS_FILES];
@@ -1897,6 +1903,21 @@ void DOS_SetupFiles (void) {
 		if (Drives[i]) DriveManager::UnmountDrive(i);
 		Drives[i]=0;
 	}
+    for (int i=0; i<MAX_DISK_IMAGES; i++) {
+        if (imageDiskList[i]) {
+            delete imageDiskList[i];
+            imageDiskList[i] = NULL;
+        }
+    }
+    if (swapInDisksSpecificDrive != -1) {
+        for (size_t si=0;si < MAX_SWAPPABLE_DISKS;si++) {
+            if (diskSwap[si] != NULL) {
+                diskSwap[si]->Release();
+                diskSwap[si] = NULL;
+            }
+        }
+        swapInDisksSpecificDrive = -1;
+    }
 	Drives[25]=new Virtual_Drive();
 }
 
@@ -1973,14 +1994,37 @@ void DOS_File::LoadState( std::istream& stream, bool pop )
 }
 
 extern bool dos_kernel_disabled;
-
+struct Alloc {
+    Bit16u bytes_sector;
+    Bit8u sectors_cluster;
+    Bit16u total_clusters;
+    Bit16u free_clusters;
+    Bit8u mediaid;
+};
+Alloc lalloc, oalloc;
+struct Opts {
+    Bit32u bytesector;
+    Bit32u cylsector;
+    Bit32u headscyl;
+    Bit32u cylinders;
+    int mounttype;
+    Bit8u mediaid;
+    unsigned char CDROM_drive;
+    unsigned long cdrom_sector_offset;
+    unsigned char floppy_emu_type;
+};
+Opts opts;
+char overlaydir[CROSS_LEN];
+void MSCDEX_SetCDInterface(int intNr, int forceCD);
 void POD_Save_DOS_Files( std::ostream& stream )
 {
+    char dinfo[256];
 	if (!dos_kernel_disabled) {
 		// 1. Do drives first (directories -> files)
 		// 2. Then files next
 
-		for( int lcv=0; lcv<DOS_DRIVES; lcv++ ) {
+		for( int i=2; i<DOS_DRIVES+2; i++ ) {
+            int lcv=i<DOS_DRIVES?i:i-DOS_DRIVES;
 			Bit8u drive_valid;
 
 			drive_valid = 0;
@@ -1992,8 +2036,49 @@ void POD_Save_DOS_Files( std::ostream& stream )
 
 			// - reloc ptr
 			WRITE_POD( &drive_valid, drive_valid );
-
 			if( drive_valid == 0xff ) continue;
+
+            strcpy(dinfo, Drives[lcv]->GetInfo());
+            WRITE_POD( &dinfo, dinfo);
+            *overlaydir=0;
+            if (!strncmp(dinfo,"local directory ",16) || !strncmp(dinfo,"CDRom ",6)) {
+                localDrive *ldp = dynamic_cast<localDrive*>(Drives[lcv]);
+                if (!ldp) ldp = dynamic_cast<cdromDrive*>(Drives[lcv]);
+                if (ldp) {
+                    lalloc.bytes_sector=ldp->allocation.bytes_sector;
+                    lalloc.sectors_cluster=ldp->allocation.sectors_cluster;
+                    lalloc.total_clusters=ldp->allocation.total_clusters;
+                    lalloc.free_clusters=ldp->allocation.free_clusters;
+                    lalloc.mediaid=ldp->allocation.mediaid;
+                }
+                Overlay_Drive *odp = dynamic_cast<Overlay_Drive*>(Drives[lcv]);
+                if (odp) {
+                    strcpy(overlaydir,odp->getOverlaydir());
+                    oalloc.bytes_sector=odp->allocation.bytes_sector;
+                    oalloc.sectors_cluster=odp->allocation.sectors_cluster;
+                    oalloc.total_clusters=odp->allocation.total_clusters;
+                    oalloc.free_clusters=odp->allocation.free_clusters;
+                    oalloc.mediaid=odp->allocation.mediaid;
+                }
+            } else if (!strncmp(dinfo,"fatDrive ",9)) {
+                fatDrive *fdp = dynamic_cast<fatDrive*>(Drives[lcv]);
+                if (fdp) {
+                    opts.bytesector=fdp->loadedDisk?fdp->loadedDisk->sector_size:fdp->opts.bytesector;
+                    opts.cylsector=fdp->loadedDisk?fdp->loadedDisk->sectors:fdp->opts.cylsector;
+                    opts.headscyl=fdp->loadedDisk?fdp->loadedDisk->heads:fdp->opts.headscyl;
+                    opts.cylinders=fdp->loadedDisk?fdp->loadedDisk->cylinders:fdp->opts.cylinders;
+                    opts.mounttype=fdp->opts.mounttype;
+                    opts.mediaid=fdp->GetMediaByte();
+                    opts.CDROM_drive=fdp->el.CDROM_drive;
+                    opts.cdrom_sector_offset=fdp->el.cdrom_sector_offset;
+                    opts.floppy_emu_type=fdp->el.floppy_emu_type;
+                }
+            }
+
+            WRITE_POD( &overlaydir, overlaydir);
+            WRITE_POD( &lalloc, lalloc);
+            WRITE_POD( &oalloc, oalloc);
+            WRITE_POD( &opts, opts);
 			Drives[lcv]->SaveState(stream);
 		}
 
@@ -2040,26 +2125,175 @@ void POD_Save_DOS_Files( std::ostream& stream )
 			WRITE_POD( &file_drive, file_drive );
 			WRITE_POD( &file_flags, file_flags );
 
-
 			Files[lcv]->SaveState(stream);
 		}
-	}
+	} else {
+        for( int i=2; i<DOS_DRIVES+2; i++ ) {
+            int lcv=i<DOS_DRIVES?i:i-DOS_DRIVES;
+			Bit8u drive_valid;
+			drive_valid = 0;
+			if( Drives[lcv] == 0 ) drive_valid = 0xff;
+			WRITE_POD( &drive_valid, drive_valid );
+			if( drive_valid == 0xff ) continue;
+            strcpy(dinfo, Drives[lcv]->GetInfo());
+            WRITE_POD( &dinfo, dinfo);
+        }
+    }
+
+    for (int i=2; i<MAX_DISK_IMAGES+2; i++) {
+        int d=i<MAX_DISK_IMAGES?i:i-MAX_DISK_IMAGES;
+        bool valid=imageDiskList[d]!=NULL;
+        WRITE_POD( &valid, valid );
+        if (!valid) continue;
+        char diskname[256];
+        imageDiskElToritoFloppy *image = dynamic_cast<imageDiskElToritoFloppy *>(imageDiskList[d]);
+        strcpy(diskname, image!=NULL?"El Torito floppy drive":imageDiskList[d]->diskname.c_str());
+        WRITE_POD( &diskname, diskname );
+        if (image!=NULL) {
+            opts.CDROM_drive=image->CDROM_drive;
+            opts.cdrom_sector_offset=image->cdrom_sector_offset;
+            opts.floppy_emu_type=image->floppy_type;
+            opts.mounttype=1;
+        } else {
+            imageDiskMemory* idmem = dynamic_cast<imageDiskMemory *>(imageDiskList[d]);
+            imageDiskVHD* idvhd = dynamic_cast<imageDiskVHD *>(imageDiskList[d]);
+            if (idmem!=NULL)
+                opts.mounttype=2;
+            else if (idvhd!=NULL)
+                opts.mounttype=3;
+            else {
+                opts.bytesector=imageDiskList[d]->sector_size;
+                opts.cylsector=imageDiskList[d]->sectors;
+                opts.headscyl=imageDiskList[d]->heads;
+                opts.cylinders=imageDiskList[d]->cylinders;
+                opts.mounttype=0;
+            }
+        }
+        WRITE_POD( &opts, opts);
+    }
 }
 
+void DOS_EnableDriveMenu(char drv);
+void IDE_Auto(signed char &index,bool &slave);
+bool AttachToBiosByLetter(imageDisk* image, const char drive);
+bool AttachToBiosAndIdeByLetter(imageDisk* image, const char drive, const unsigned char ide_index, const bool ide_slave);
+imageDiskMemory* CreateRamDrive(Bitu sizes[], const int reserved_cylinders, const bool forceFloppy, Program* obj);
+
+void unmount(int lcv) {
+    if (!Drives[lcv] || lcv>=DOS_DRIVES-1) return;
+    const isoDrive* cdrom = dynamic_cast<isoDrive*>(Drives[lcv]);
+    if (DriveManager::UnmountDrive(lcv) == 0) {
+        if (cdrom) IDE_CDROM_Detach(lcv);
+        Drives[lcv]=0;
+        DOS_EnableDriveMenu('A'+lcv);
+        mem_writeb(Real2Phys(dos.tables.mediaid)+(unsigned int)'A'+lcv*dos.tables.dpb_size,0);
+    }
+}
 
 void POD_Load_DOS_Files( std::istream& stream )
 {
+    char dinfo[256];
+    std::vector<int> clist;
+    clist.clear();
 	if (!dos_kernel_disabled) {
 		// 1. Do drives first (directories -> files)
 		// 2. Then files next
 
-		for( int lcv=0; lcv<DOS_DRIVES; lcv++ ) {
+		for( int i=2; i<DOS_DRIVES+2; i++ ) {
+            int lcv=i<DOS_DRIVES?i:i-DOS_DRIVES;
 			Bit8u drive_valid;
 
 			// - reloc ptr
 			READ_POD( &drive_valid, drive_valid );
-			if( drive_valid == 0xff ) continue;
+			if( drive_valid == 0xff ) {
+                if (Drives[lcv] && lcv<DOS_DRIVES-1) unmount(lcv);
+                continue;
+            }
 
+            READ_POD( &dinfo, dinfo);
+            READ_POD( &overlaydir, overlaydir);
+            READ_POD( &lalloc, lalloc);
+            READ_POD( &oalloc, oalloc);
+            READ_POD( &opts, opts);
+            if( Drives[lcv] && strcasecmp(Drives[lcv]->info, dinfo) && (!strncmp(dinfo,"local directory ",16) || !strncmp(dinfo,"CDRom ",6) || (!strncmp(dinfo,"isoDrive ",9) || !strncmp(dinfo,"fatDrive ",9))))
+                unmount(lcv);
+            if( !Drives[lcv] ) {
+                std::vector<std::string> options;
+                if (!strncmp(dinfo,"local directory ",16)) {
+                    Drives[lcv]=new localDrive(dinfo+16,lalloc.bytes_sector,lalloc.sectors_cluster,lalloc.total_clusters,lalloc.free_clusters,lalloc.mediaid,options);
+                    if (Drives[lcv]) {
+                        DOS_EnableDriveMenu('A'+lcv);
+                        mem_writeb(Real2Phys(dos.tables.mediaid)+lcv*dos.tables.dpb_size,lalloc.mediaid);
+                        if (strlen(overlaydir)) {
+                            Bit8u o_error = 0;
+                            Drives[lcv]=new Overlay_Drive(dynamic_cast<localDrive*>(Drives[lcv])->getBasedir(),overlaydir,oalloc.bytes_sector,oalloc.sectors_cluster,oalloc.total_clusters,oalloc.free_clusters,oalloc.mediaid,o_error,options);
+                        }
+                    } else
+                        LOG_MSG("Error: Cannot restore drive from directory %s\n", dinfo+16);
+                } else if (!strncmp(dinfo,"CDRom ",6)) {
+                    int num = -1;
+                    int error;
+                    int id, major, minor;
+                    DOSBox_CheckOS(id, major, minor);
+                    if ((id==VER_PLATFORM_WIN32_NT) && (major>5))
+                        MSCDEX_SetCDInterface(CDROM_USE_IOCTL_DX, num);
+                    else
+                        MSCDEX_SetCDInterface(CDROM_USE_IOCTL_DIO, num);
+                    Drives[lcv] = new cdromDrive('A'+lcv,dinfo+6,lalloc.bytes_sector,lalloc.sectors_cluster,lalloc.total_clusters,lalloc.free_clusters,lalloc.mediaid,error,options);
+                    if (Drives[lcv]) {
+                        DOS_EnableDriveMenu('A'+lcv);
+                        mem_writeb(Real2Phys(dos.tables.mediaid)+lcv*dos.tables.dpb_size,lalloc.mediaid);
+                    } else
+                        LOG_MSG("Error: Cannot restore drive from directory %s\n", dinfo+6);
+                } else if (!strncmp(dinfo,"isoDrive ",9) && *(dinfo+9)) {
+                    MSCDEX_SetCDInterface(CDROM_USE_SDL, -1);
+                    Bit8u mediaid = 0xF8;
+                    int error = -1;
+                    isoDrive* newDrive = new isoDrive('A'+lcv, dinfo+9, mediaid, error);
+                    if (!error) {
+                        Drives[lcv] = newDrive;
+                        DriveManager::AppendDisk(lcv, newDrive);
+                        DriveManager::InitializeDrive(lcv);
+                        DOS_EnableDriveMenu('A'+lcv);
+                        mem_writeb(Real2Phys(dos.tables.mediaid) + lcv*dos.tables.dpb_size, mediaid);
+                        clist.push_back(lcv);
+                    }
+                } else if (!strncmp(dinfo,"fatDrive ",9)) {
+                    fatDrive* newDrive = NULL;
+                    Bitu sizes[4] = { 0,0,0,0 };
+                    if (opts.mounttype==1) {
+                        imageDisk * newImage = new imageDiskElToritoFloppy((unsigned char)opts.CDROM_drive, opts.cdrom_sector_offset, opts.floppy_emu_type);
+                        if (newImage != NULL) {
+                            newImage->Addref();
+                            newDrive = new fatDrive(newImage, options);
+                            newImage->Release();
+                        } else
+                            LOG_MSG("Error: Cannot restore drive from El Torito floppy image.\n");
+
+                    } else if (opts.mounttype==2) {
+                        imageDiskMemory* image = CreateRamDrive(sizes, 0, lcv < 2 && sizes[0] == 0, NULL);
+                        if (image != NULL && image->Format() == 0x00) {
+                            image->Addref();
+                            newDrive = new fatDrive(image, options);
+                            image->Release();
+                        }
+                    } else if (opts.mounttype==3 && *(dinfo+9)) {
+                        imageDisk* vhdImage = NULL;
+                        if (imageDiskVHD::Open(dinfo+9, false, &vhdImage)==imageDiskVHD::OPEN_SUCCESS)
+                            newDrive = new fatDrive(vhdImage, options);
+                        vhdImage = NULL;
+                    } else if (!opts.mounttype && *(dinfo+9))
+                        newDrive = new fatDrive(dinfo+9, opts.bytesector, opts.cylsector, opts.headscyl, opts.cylinders, options);
+                    if (newDrive && newDrive->created_successfully) {
+                        Drives[lcv] = newDrive;
+                        DriveManager::AppendDisk(lcv, newDrive);
+                        DriveManager::InitializeDrive(lcv);
+                        DOS_EnableDriveMenu('A'+lcv);
+                        mem_writeb(Real2Phys(dos.tables.mediaid) + lcv*dos.tables.dpb_size, opts.mediaid);
+                    } else if ((!opts.mounttype || opts.mounttype==3) && *(dinfo+9))
+                        LOG_MSG("Error: Cannot restore drive from image file %s\n", dinfo+9);
+                }
+            }
 			if( Drives[lcv] ) Drives[lcv]->LoadState(stream);
 		}
 
@@ -2135,5 +2369,102 @@ void POD_Load_DOS_Files( std::istream& stream )
 		if (dummy != NULL) {
 			delete dummy;
 		}
-	}
+	} else {
+        for( int i=2; i<DOS_DRIVES+2; i++ ) {
+            int lcv=i<DOS_DRIVES?i:i-DOS_DRIVES;
+			Bit8u drive_valid;
+
+			READ_POD( &drive_valid, drive_valid );
+			if( drive_valid == 0xff ) {
+                if (Drives[lcv]) unmount(lcv);
+                continue;
+            }
+            READ_POD( &dinfo, dinfo);
+            if( Drives[lcv] && strcasecmp(Drives[lcv]->info, dinfo))
+                unmount(lcv);
+            if (!Drives[lcv] && !strncmp(dinfo,"isoDrive ",9)) {
+                MSCDEX_SetCDInterface(CDROM_USE_SDL, -1);
+                Bit8u mediaid = 0xF8;
+                int error = -1;
+                isoDrive* newDrive = new isoDrive('A'+lcv, dinfo+9, mediaid, error);
+                if (!error) {
+                    Drives[lcv] = newDrive;
+                    DriveManager::AppendDisk(lcv, newDrive);
+                    DriveManager::InitializeDrive(lcv);
+                    mem_writeb(Real2Phys(dos.tables.mediaid) + lcv*dos.tables.dpb_size, mediaid);
+                    clist.push_back(lcv);
+                }
+            }
+        }
+    }
+
+    for (int i=2; i<MAX_DISK_IMAGES+2; i++) {
+        int d=i<MAX_DISK_IMAGES?i:i-MAX_DISK_IMAGES;
+        bool ide_slave = false;
+        signed char ide_index = -1;
+        bool valid=false;
+        READ_POD( &valid, valid );
+        if (!valid) {
+            if (imageDiskList[d]) {
+                if (d > 1) IDE_Hard_Disk_Detach(d);
+                imageDiskList[d]->Release();
+                imageDiskList[d] = NULL;
+                imageDiskChange[d] = true;
+            }
+            if (i==MAX_DISK_IMAGES-1)
+                for (auto &it : clist) {
+                    ide_slave = false;
+                    ide_index = -1;
+                    IDE_Auto(ide_index,ide_slave);
+                    if (ide_index >= 0) IDE_CDROM_Attach(ide_index, ide_slave, it);
+                }
+            continue;
+        }
+        char diskname[256];
+        READ_POD( &diskname, diskname );
+        READ_POD( &opts, opts);
+        Bitu sizes[4] = { 0,0,0,0 };
+        ide_slave = false;
+        ide_index = -1;
+        IDE_Auto(ide_index,ide_slave);
+        if( imageDiskList[d] && ((opts.mounttype==1 && dynamic_cast<imageDiskElToritoFloppy *>(imageDiskList[d])==NULL) || (opts.mounttype!=1 && strcasecmp(imageDiskList[d]->diskname.c_str(), diskname )))) {
+            if (d > 1) IDE_Hard_Disk_Detach(d);
+            imageDiskList[d]->Release();
+            imageDiskList[d] = NULL;
+            imageDiskChange[d] = true;
+        }
+        if (imageDiskList[d])
+            ;
+        else if (opts.mounttype==1) {
+            imageDisk * image = new imageDiskElToritoFloppy((unsigned char)opts.CDROM_drive, opts.cdrom_sector_offset, opts.floppy_emu_type);
+            if (image) AttachToBiosByLetter(image, 'A'+d);
+            else LOG_MSG("Warning: Cannot restore drive number from El Torito floppy image.\n");
+        } else if (opts.mounttype==2) {
+            imageDiskMemory* image = CreateRamDrive(sizes, 0, d < 2 && sizes[0] == 0, NULL);
+            if (image != NULL && image->Format() == 0x00) AttachToBiosAndIdeByLetter(image, d, (unsigned char)ide_index, ide_slave);
+        } else if (opts.mounttype==3 && *diskname) {
+            imageDisk* image = NULL;
+            if (imageDiskVHD::Open(diskname, false, &image)==imageDiskVHD::OPEN_SUCCESS)
+                AttachToBiosAndIdeByLetter(image, 'A'+d, (unsigned char)ide_index, ide_slave);
+            else
+                LOG_MSG("Warning: Cannot restore drive number from image file %s\n", diskname);
+            image = NULL;
+        } else if (!opts.mounttype && *diskname) {
+            std::vector<std::string> options;
+            fatDrive* newDrive = new fatDrive(diskname, opts.bytesector, opts.cylsector, opts.headscyl, opts.cylinders, options);
+            if (newDrive->created_successfully) {
+                imageDisk* image = newDrive->loadedDisk;
+                AttachToBiosAndIdeByLetter(image, 'A'+d, (unsigned char)ide_index, ide_slave);
+            } else
+                LOG_MSG("Warning: Cannot restore drive number from image file %s\n", diskname);
+            if (newDrive) delete newDrive;
+        }
+        if (i==MAX_DISK_IMAGES-1)
+            for (auto &it : clist) {
+                ide_slave = false;
+                ide_index = -1;
+                IDE_Auto(ide_index,ide_slave);
+                if (ide_index >= 0) IDE_CDROM_Attach(ide_index, ide_slave, it);
+            }
+    }
 }
