@@ -1408,7 +1408,9 @@ void CMscdex::LoadState( std::istream& stream )
 void POD_Save_DOS_Mscdex( std::ostream& stream )
 {
 	if (!dos_kernel_disabled) {
-		for (Bit8u drive_unit=0; drive_unit<mscdex->GetNumDrives(); drive_unit++) {
+		Bit16u dnum=mscdex->GetNumDrives();
+		WRITE_POD( &dnum, dnum);
+		for (Bit8u drive_unit=0; drive_unit<dnum; drive_unit++) {
 			TMSF pos, start, end;
 			bool playing, pause;
 
@@ -1431,7 +1433,16 @@ void POD_Save_DOS_Mscdex( std::ostream& stream )
 void POD_Load_DOS_Mscdex( std::istream& stream )
 {
 	if (!dos_kernel_disabled) {
-		for (Bit8u drive_unit=0; drive_unit<mscdex->GetNumDrives(); drive_unit++) {
+		Bit16u dnum;
+		READ_POD( &dnum, dnum);
+        if (mscdex->GetNumDrives()>dnum) {
+            mscdex->numDrives=dnum;
+            for (Bit16u i=dnum; i<mscdex->GetNumDrives(); i++) {
+                delete mscdex->cdrom[i];
+                mscdex->cdrom[i] = 0;
+            }
+        }
+		for (Bit8u drive_unit=0; drive_unit<dnum; drive_unit++) {
 			TMSF pos, start, end;
 			Bit32u msf_time, play_len;
 			bool playing, pause;
