@@ -38,6 +38,7 @@ public:
 	static void AppendDisk(int drive, DOS_Drive* disk);
 	static void InitializeDrive(int drive);
 	static int UnmountDrive(int drive);
+	static char * GetDrivePosition(int drive);
 //	static void CycleDrive(bool pressed);
 //	static void CycleDisk(bool pressed);
 	static void CycleDisks(int drive, bool notify);
@@ -96,7 +97,13 @@ public:
 	virtual void EmptyCache(void) { dirCache.EmptyCache(); };
 	virtual void MediaChange() {};
 	const char* getBasedir() {return basedir;};
-
+	struct {
+		Bit16u bytes_sector;
+		Bit8u sectors_cluster;
+		Bit16u total_clusters;
+		Bit16u free_clusters;
+		Bit8u mediaid;
+	} allocation;
 	int remote = -1;
 
 protected:
@@ -106,14 +113,6 @@ protected:
 	struct {
 		char srch_dir[CROSS_LEN];
     } srchInfo[MAX_OPENDIRS] = {};
-
-	struct {
-		Bit16u bytes_sector;
-		Bit8u sectors_cluster;
-		Bit16u total_clusters;
-		Bit16u free_clusters;
-		Bit8u mediaid;
-	} allocation;
 };
 
 #if 0 // nothing uses this
@@ -413,6 +412,19 @@ public:
 	imageDisk *loadedDisk = NULL;
 	uint8_t req_ver_major = 0,req_ver_minor = 0;
 	bool created_successfully = true;
+    struct {
+        Bit32u bytesector;
+        Bit32u cylsector;
+        Bit32u headscyl;
+        Bit32u cylinders;
+        int mounttype;
+    } opts = {0, 0, 0, 0, -1};
+    struct {
+        unsigned char CDROM_drive;
+        unsigned long cdrom_sector_offset;
+        unsigned char floppy_emu_type;
+    } el = {0, 0, 0};
+
 private:
 	char* Generate_SFN(const char *path, const char *name);
 	Bit32u getClusterValue(Bit32u clustNum);
