@@ -325,6 +325,7 @@ static const char *def_menu_video_pc98[] =
     NULL
 };
 
+#if C_DEBUG
 /* video output debug ("VideoDebugMenu") */
 static const char *def_menu_video_debug[] =
 {
@@ -334,6 +335,7 @@ static const char *def_menu_video_debug[] =
     "debug_retracepoll",
     NULL
 };
+#endif
 
 /* video menu ("VideoMenu") */
 static const char *def_menu_video[] =
@@ -400,6 +402,9 @@ static const char *def_menu_dos[] =
     "DOSWinMenu",
 #endif
     "shell_config_commands",
+#if defined(WIN32) && !defined(HX_DOS)
+    "quick_launch",
+#endif
     "--",
     "quick_reboot",
     "--",
@@ -407,8 +412,14 @@ static const char *def_menu_dos[] =
     "mapper_swapcd",
     "--",
     "mapper_rescanall",
-#if C_DEBUG
+    "list_drivenum",
+#if C_PRINTER || C_DEBUG
     "--",
+#endif
+#if C_PRINTER
+    "mapper_ejectpage",
+#endif
+#if C_DEBUG
     "DOSDebugMenu",
 #endif
     NULL
@@ -464,6 +475,7 @@ static const char *def_menu_dos_win[] =
 };
 #endif
 
+#if C_DEBUG
 /* DOS debug ("DOSDebugMenu") */
 static const char *def_menu_dos_debug[] =
 {
@@ -471,6 +483,7 @@ static const char *def_menu_dos_debug[] =
     "debug_logfileio",
     NULL
 };
+#endif
 
 /* sound menu ("SoundMenu") */
 static const char *def_menu_sound[] =
@@ -510,7 +523,7 @@ static const char *def_menu_capture[] =
     "force_loadstate",
     "mapper_savestate",
     "mapper_loadstate",
-	"saveslotmenu",
+    "saveslotmenu",
     NULL
 };
 
@@ -531,9 +544,13 @@ static const char *def_menu_capture_format[] =
 /* Save slots */
 static const char *def_save_slots[] =
 {
-	"mapper_prevslot",
-	"mapper_nextslot",
-	"--",
+    "current_page",
+    "prev_page",
+    "next_page",
+    "--",
+    "first_page",
+    "last_page",
+    "--",
     "slot0",
     "slot1",
     "slot2",
@@ -544,9 +561,14 @@ static const char *def_save_slots[] =
     "slot7",
     "slot8",
     "slot9",
-	"--",
+    "--",
+    "mapper_prevslot",
+    "mapper_nextslot",
+    "--",
+    "removestate",
+    "--",
     "refreshslot",
-	NULL
+    NULL
 };
 
 /* Drive menu ("DriveMenu") */
@@ -1279,8 +1301,10 @@ void ConstructMenu(void) {
     /* video PC-98 menu */
     ConstructSubMenu(mainMenu.get_item("VideoPC98Menu").get_master_id(), def_menu_video_pc98);
 
+#if C_DEBUG
     /* video debug menu */
     ConstructSubMenu(mainMenu.get_item("VideoDebugMenu").get_master_id(), def_menu_video_debug);
+#endif
 
     /* sound menu */
     ConstructSubMenu(mainMenu.get_item("SoundMenu").get_master_id(), def_menu_sound);
@@ -1305,8 +1329,10 @@ void ConstructMenu(void) {
     ConstructSubMenu(mainMenu.get_item("DOSWinMenu").get_master_id(), def_menu_dos_win);
 #endif
 
+#if C_DEBUG
     /* DOS debug menu */
     ConstructSubMenu(mainMenu.get_item("DOSDebugMenu").get_master_id(), def_menu_dos_debug);
+#endif
 
 #if !defined(C_EMSCRIPTEN)
     /* capture menu */
