@@ -35,6 +35,8 @@
 #include "control.h"
 #include "shell.h"
 #include "cpu.h"
+#include "bios_disk.h"
+#include "../dos/drives.h"
 
 #include <iostream>
 #include <sstream>
@@ -68,7 +70,9 @@ extern unsigned char        GFX_Gshift;
 extern uint32_t             GFX_Bmask;
 extern unsigned char        GFX_Bshift;
 
-extern bool                 dos_kernel_disabled;
+extern std::string          saveloaderr;
+extern int                  statusdrive, swapInDisksSpecificDrive;
+extern bool                 dos_kernel_disabled, confres, quit_confirm;
 extern Bitu                 currentWindowWidth, currentWindowHeight;
 
 extern bool                 MSG_Write(const char *);
@@ -1430,6 +1434,129 @@ public:
     }
 };
 
+class ShowDriveInfo : public GUI::ToplevelWindow {
+protected:
+    GUI::Input *name;
+public:
+    ShowDriveInfo(GUI::Screen *parent, int x, int y, const char *title) :
+        ToplevelWindow(parent, x, y, 400, 280, title) {
+            char name[DOS_NAMELENGTH_ASCII],lname[LFN_NAMELENGTH];
+            Bit32u size;Bit16u date;Bit16u time;Bit8u attr;
+            /* Command uses dta so set it to our internal dta */
+            RealPt save_dta = dos.dta();
+            dos.dta(dos.tables.tempdta);
+            DOS_DTA dta(dos.dta());
+            if (Drives[statusdrive]) {
+                char root[7] = {(char)('A'+statusdrive),':','\\','*','.','*',0};
+                bool ret = DOS_FindFirst(root,DOS_ATTR_VOLUME);
+                if (ret) {
+                    dta.GetResult(name,lname,size,date,time,attr);
+                    DOS_FindNext(); //Mark entry as invalid
+                } else name[0] = 0;
+
+                /* Change 8.3 to 11.0 */
+                const char* dot = strchr(name, '.');
+                if(dot && (dot - name == 8) ) {
+                    name[8] = name[9];name[9] = name[10];name[10] = name[11];name[11] = 0;
+                }
+
+                root[3] = 0; //This way, the format string can be reused.
+                std::string type, path, swappos="-", overlay="-";
+                bool readonly=false;
+                const char *info = Drives[statusdrive]->GetInfo();
+                if (!strncmp(info, "fatDrive ", 9) || !strncmp(info, "isoDrive ", 9)) {
+                    type=strncmp(info, "isoDrive ", 9)?"fatDrive":"isoDrive";
+                    path=info+9;
+                    if (type=="isoDrive")
+                        readonly=true;
+                    else {
+                        readonly=Drives[statusdrive]->readonly;
+                        if (!path.size()) {
+                            fatDrive *fdp = dynamic_cast<fatDrive*>(Drives[statusdrive]);
+                            if (fdp!=NULL&&fdp->opts.mounttype==1)
+                                path="El Torito floppy drive";
+                            else if (fdp!=NULL&&fdp->opts.mounttype==2)
+                                path="RAM drive";
+                        }
+                    }
+                    swappos=DriveManager::GetDrivePosition(statusdrive);
+                } else if (!strncmp(info, "local directory ", 16)) {
+                    type="local directory";
+                    path=info+16;
+                    readonly=Drives[statusdrive]->readonly;
+                    Overlay_Drive *ddp = dynamic_cast<Overlay_Drive*>(Drives[statusdrive]);
+                    if (ddp!=NULL) {
+                        readonly=ddp->ovlreadonly;
+                        overlay=ddp->getOverlaydir();
+                    }
+                } else if (!strncmp(info, "CDRom ", 6)) {
+                    type="CDRom";
+                    path=info+6;
+                    readonly=true;
+                } else {
+                    type=info;
+                    path="";
+                    readonly=true;
+                }
+                if (path=="") path="-";
+                new GUI::Label(this, 40, 25, "Drive root: "+std::string(root));
+                new GUI::Label(this, 40, 50, "Drive type: "+type);
+                new GUI::Label(this, 40, 75, "Mounted as: "+path);
+                new GUI::Label(this, 40, 100, "Overlay at: "+overlay);
+                new GUI::Label(this, 40, 125, "Disk label: "+std::string(name));
+                new GUI::Label(this, 40, 150, "Read only : "+std::string(readonly?"Yes":"No"));
+                new GUI::Label(this, 40, 175, "Swap slot : "+swappos);
+            }
+            dos.dta(save_dta);
+            (new GUI::Button(this, 165, 205, "Close", 70))->addActionHandler(this);
+    }
+
+    void actionExecuted(GUI::ActionEventSource *b, const GUI::String &arg) {
+        (void)b;//UNUSED
+        if (arg == "Close")
+            close();
+        if (shortcut) running = false;
+    }
+};
+
+char * GetIDEPosition(unsigned char bios_disk_index);
+class ShowDriveNumber : public GUI::ToplevelWindow {
+protected:
+    GUI::Input *name;
+public:
+    ShowDriveNumber(GUI::Screen *parent, int x, int y, const char *title) :
+        ToplevelWindow(parent, x, y, 480, 260, title) {
+        std::string str;
+		for (int index = 0; index < MAX_DISK_IMAGES; index++) {
+			if (imageDiskList[index]) {
+                int swaps=0;
+                if (swapInDisksSpecificDrive == index) {
+                    for (size_t si=0;si < MAX_SWAPPABLE_DISKS;si++)
+                        if (diskSwap[si] != NULL)
+                            swaps++;
+                }
+                if (!swaps) swaps=1;
+                if (index<2)
+                    str = "Swap position: " + std::to_string(swaps==1?1:swapPosition+1) + "/" + std::to_string(swaps) + " - " + (dynamic_cast<imageDiskElToritoFloppy *>(imageDiskList[index])!=NULL?"El Torito floppy drive":imageDiskList[index]->diskname);
+                else {
+                    str = GetIDEPosition(index);
+                    str = "IDE controller: " + (str.size()?str:"NA") + " - " + imageDiskList[index]->diskname;
+                }
+            } else
+                str = "Not yet mounted";
+            new GUI::Label(this, 40, 25*(index+1), std::to_string(index) + " - " + str);
+        }
+        (new GUI::Button(this, 190, 25*(MAX_DISK_IMAGES+1)+5, "Close", 70))->addActionHandler(this);
+    }
+
+    void actionExecuted(GUI::ActionEventSource *b, const GUI::String &arg) {
+        (void)b;//UNUSED
+        if (arg == "Close")
+            close();
+        if (shortcut) running = false;
+    }
+};
+
 class ShowStateCorrupt : public GUI::ToplevelWindow {
 protected:
     GUI::Input *name;
@@ -1448,6 +1575,53 @@ public:
     }
 };
 
+class ShowLoadWarning : public GUI::ToplevelWindow {
+protected:
+    GUI::Input *name;
+public:
+    ShowLoadWarning(GUI::Screen *parent, int x, int y, const char *title) :
+        ToplevelWindow(parent, x, y, 430, 120, "Warning") {
+            new GUI::Label(this, strncmp(title, "DOSBox-X ", 9)?30:10, 20, title);
+            (new GUI::Button(this, 140, 50, "Yes", 70))->addActionHandler(this);
+            (new GUI::Button(this, 230, 50, "No", 70))->addActionHandler(this);
+    }
+
+    void actionExecuted(GUI::ActionEventSource *b, const GUI::String &arg) {
+        (void)b;//UNUSED
+        if (arg == "Yes")
+            confres=true;
+        if (arg == "No")
+            confres=false;
+        close();
+        if (shortcut) running = false;
+    }
+};
+
+class ShowQuitWarning : public GUI::ToplevelWindow {
+protected:
+    GUI::Input *name;
+public:
+    ShowQuitWarning(GUI::Screen *parent, int x, int y, const char *title) :
+        ToplevelWindow(parent, x, y, strcmp(title, "quit1")?430:330, !strcmp(title, "quit3")?180:150, "Quit DOSBox-X warning") {
+            bool forcequit=!strcmp(title, "quit3");
+            new GUI::Label(this, forcequit?20:40, 20, !strcmp(title, "quit1")?"This will quit from DOSBox-X.":(!strcmp(title, "quit2")?"You are currently running a guest system.":"It may be unsafe to quit from DOSBox-X right now"));
+            if (forcequit) new GUI::Label(this, forcequit?20:40, 50, "because one or more files are currently open.");
+            new GUI::Label(this, forcequit?20:40, forcequit?80:50, strcmp(title, "quit1")?"Are you sure to quit anyway now?":"Are you sure?");
+            (new GUI::Button(this, strcmp(title, "quit1")?140:90, forcequit?110:80, "Yes", 70))->addActionHandler(this);
+            (new GUI::Button(this, strcmp(title, "quit1")?230:180, forcequit?110:80, "No", 70))->addActionHandler(this);
+    }
+
+    void actionExecuted(GUI::ActionEventSource *b, const GUI::String &arg) {
+        (void)b;//UNUSED
+        if (arg == "Yes")
+            quit_confirm=true;
+        if (arg == "No")
+            quit_confirm=false;
+        close();
+        if (shortcut) running = false;
+    }
+};
+
 class ConfigurationWindow : public GUI::ToplevelWindow {
 public:
     GUI::Button *saveButton, *closeButton;
@@ -1765,6 +1939,50 @@ static void UI_Select(GUI::ScreenSDL *screen, int select) {
             auto *np6 = new ShowStateCorrupt(screen, 150, 120, "Save state corrupted! Program may not work.");
             np6->raise();
             } break;
+        case 22: if (saveloaderr.size()) {
+            auto *np6 = new ShowStateCorrupt(screen, 150, 120, saveloaderr.c_str());
+            np6->raise();
+            } break;
+        case 23: {
+            auto *np7 = new ShowLoadWarning(screen, 150, 120, "DOSBox-X version mismatch. Load the state anyway?");
+            np7->raise();
+            } break;
+        case 24: {
+            auto *np7 = new ShowLoadWarning(screen, 150, 120, "Program name mismatch. Load the state anyway?");
+            np7->raise();
+            } break;
+        case 25: {
+            auto *np7 = new ShowLoadWarning(screen, 150, 120, "Memory size mismatch. Load the state anyway?");
+            np7->raise();
+            } break;
+        case 26: {
+            auto *np7 = new ShowLoadWarning(screen, 150, 120, "Machine type mismatch. Load the state anyway?");
+            np7->raise();
+            } break;
+        case 27: {
+            auto *np7 = new ShowLoadWarning(screen, 150, 120, "Are you sure to remove the state in this slot?");
+            np7->raise();
+            } break;
+        case 28: {
+            auto *np8 = new ShowQuitWarning(screen, 150, 120, "quit1");
+            np8->raise();
+            } break;
+        case 29: {
+            auto *np8 = new ShowQuitWarning(screen, 120, 100, "quit2");
+            np8->raise();
+            } break;
+        case 30: {
+            auto *np8 = new ShowQuitWarning(screen, 120, 100, "quit3");
+            np8->raise();
+            } break;
+        case 31: if (statusdrive>-1 && statusdrive<DOS_DRIVES && Drives[statusdrive]) {
+            auto *np9 = new ShowDriveInfo(screen, 120, 50, "Drive Information");
+            np9->raise();
+            } break;
+        case 32: {
+            auto *np10 = new ShowDriveNumber(screen, 110, 70, "Mounted Drive Numbers");
+            np10->raise();
+            } break;
         default:
             break;
     }
