@@ -252,42 +252,103 @@ void CPrinter::updateFont()
 	if (curFont != NULL)
 		FT_Done_Face(curFont);
 
-	const char* fontName;
+	std::string fontName, basedir;
+#if defined(WIN32)
+    basedir = ".\\FONTS\\";
+#else
+    basedir = "./FONTS/";
+#endif
 
 	switch (LQtypeFace)
 	{
 	    case roman:
-		    fontName = "./FONTS/roman.ttf";
+		    fontName = basedir + "roman.ttf";
 		    break;
 	    case sansserif:
-		    fontName = "./FONTS/sansserif.ttf";
+		    fontName = basedir + "sansserif.ttf";
 		    break;
 	    case courier:
-		    fontName = "./FONTS/courier.ttf";
+		    fontName = basedir + "courier.ttf";
 		    break;
 	    case script:
-		    fontName = "./FONTS/script.ttf";
+		    fontName = basedir + "script.ttf";
 		    break;
 	    case ocra:
 	    case ocrb:
-		    fontName = "./FONTS/ocra.ttf";
+		    fontName = basedir + "ocra.ttf";
 		    break;
 	    default:
-		    fontName = "./FONTS/roman.ttf";
+		    fontName = basedir + "roman.ttf";
 	}
 	
 #ifndef WIN32
 	std::string configfont;
 	Cross::GetPlatformConfigDir(configfont);
 	configfont += fontName;
-	fontName = configfont.c_str();
+	fontName = configfont;
 #endif
 	
-	if (FT_New_Face(FTlib, fontName, 0, &curFont))
+	if (FT_New_Face(FTlib, fontName.c_str(), 0, &curFont))
 	{
-		//LOG(LOG_MISC,LOG_ERROR)("Unable to load font %s", fontName);
-		LOG_MSG("Unable to load font %s", fontName);
-		curFont = NULL;
+        std::string oldfont=fontName;
+        struct stat wstat;
+#if defined(WIN32)
+        const char* windir = "C:\\WINDOWS";
+        if(stat(windir,&wstat) || !(wstat.st_mode & S_IFDIR)) {
+            TCHAR dir[MAX_PATH];
+            if (GetWindowsDirectory(dir, MAX_PATH))
+                windir=dir;
+        }
+        basedir = std::string(windir) + "\\fonts\\";
+#else
+        basedir = "/usr/share/fonts/";
+#endif
+        switch (LQtypeFace)
+        {
+            case roman:
+#if defined(WIN32)
+                fontName = basedir + "times.ttf";
+#else
+                fontName = basedir + "liberation-serif/LiberationSerif-Regular.ttf";
+                if(stat(fontName.c_str(),&wstat)) fontName = basedir + "liberation/LiberationSerif-Regular.ttf";
+#endif
+                break;
+            case sansserif:
+#if defined(WIN32)
+                fontName = basedir + "arial.ttf";
+#else
+                fontName = basedir + "liberation-sans/LiberationSans-Regular.ttf";
+                if(stat(fontName.c_str(),&wstat)) fontName = basedir + "liberation/LiberationSans-Regular.ttf";
+#endif
+                break;
+            case courier:
+#if defined(WIN32)
+                fontName = basedir + "cour.ttf";
+#else
+                fontName = basedir + "liberation-mono/LiberationMono-Regular.ttf";
+                if(stat(fontName.c_str(),&wstat)) fontName = basedir + "liberation/LiberationMono-Regular.ttf";
+#endif
+                break;
+            case script:
+                fontName = basedir + "freescpt.ttf";
+                break;
+            case ocra:
+            case ocrb:
+                fontName = basedir + "ocraext.ttf";
+                break;
+            default:
+#if defined(WIN32)
+                fontName = basedir + "times.ttf";
+#else
+                fontName = basedir + "liberation-serif/LiberationSerif-Regular.ttf";
+                if(stat(fontName.c_str(),&wstat)) fontName = basedir + "liberation/LiberationSerif-Regular.ttf";
+#endif
+        }
+        if (FT_New_Face(FTlib, fontName.c_str(), 0, &curFont)) {
+            //LOG(LOG_MISC,LOG_ERROR)("Unable to load font %s", fontName);
+            LOG_MSG("Unable to load font %s (or %s)", oldfont.c_str(), fontName.c_str());
+            curFont = NULL;
+        }
 	}
 
 	Real64 horizPoints = 10.5;
@@ -1597,7 +1658,7 @@ static void findNextName(const char* front, const char* ext, char* fname)
 
 void CPrinter::outputPage() 
 {
-	char fname[200]; 
+	char fname[200];
 
 	if (strcasecmp(output, "printer") == 0)
 	{
@@ -2138,7 +2199,9 @@ void PRINTER_Init()
 	//IO_RegisterWriteHandler(LPTPORT+2,PRINTER_writecontrol,IO_MB);
 	//IO_RegisterReadHandler(LPTPORT+2,PRINTER_readcontrol,IO_MB);
 
-	MAPPER_AddHandler(FormFeed, MK_f2 , MMOD1, "ejectpage", "Formfeed");
+    DOSBoxMenu::item *item;
+	MAPPER_AddHandler(FormFeed, MK_f2 , MMOD1, "ejectpage", "FormFeed", &item);
+    item->set_text("Send form-feed");
 }
 
 #endif
