@@ -38,7 +38,7 @@ Bitu call_program;
 
 extern int enablelfn, paste_speed, wheel_key;
 extern const char *modifier;
-extern bool dos_kernel_disabled, force_nocachedir, freesizecap, wpcolon, enable_config_as_shell_commands, load, startwait, mountwarning;
+extern bool dos_kernel_disabled, force_nocachedir, freesizecap, wpcolon, enable_config_as_shell_commands, load, winrun, winautorun, startwait, mountwarning;
 
 /* This registers a file on the virtual drive and creates the correct structure for it*/
 
@@ -1079,14 +1079,15 @@ void CONFIG::Run(void) {
 						} else if (!strcasecmp(pvars[0].c_str(), "dos")) {
 							mountwarning = section->Get_bool("mountwarning");
 							if (!strcasecmp(inputline.substr(0, 4).c_str(), "lfn=")) {
-								if (!strcmp(section->Get_string("lfn"), "true")) enablelfn=1;
-								else if (!strcmp(section->Get_string("lfn"), "false")) enablelfn=0;
-								else if (!strcmp(section->Get_string("lfn"), "autostart")) enablelfn=-2;
+								std::string lfn = section->Get_string("lfn");
+								if (lfn=="true"||lfn=="1") enablelfn=1;
+								else if (lfn=="false"||lfn=="0") enablelfn=0;
+								else if (lfn=="autostart") enablelfn=-2;
 								else enablelfn=-1;
 								mainMenu.get_item("dos_lfn_auto").check(enablelfn==-1).refresh_item(mainMenu);
 								mainMenu.get_item("dos_lfn_enable").check(enablelfn==1).refresh_item(mainMenu);
 								mainMenu.get_item("dos_lfn_disable").check(enablelfn==0).refresh_item(mainMenu);
-								uselfn = enablelfn==1 || ((enablelfn == -1 || enablelfn == -2) && dos.version.major>6);
+								uselfn = enablelfn==1 || ((enablelfn == -1 || enablelfn == -2) && (dos.version.major>6 || winrun));
 							} else if (!strcasecmp(inputline.substr(0, 4).c_str(), "ver=")) {
 								char *ver = (char *)section->Get_string("ver");
 								if (!*ver) {
@@ -1099,6 +1100,9 @@ void CONFIG::Run(void) {
 								enable_config_as_shell_commands = section->Get_bool("shell configuration as commands");
 								mainMenu.get_item("shell_config_commands").check(enable_config_as_shell_commands).enable(true).refresh_item(mainMenu);
 #if defined(WIN32) && !defined(HX_DOS)
+							} else if (!strcasecmp(inputline.substr(0, 9).c_str(), "startcmd=")) {
+								winautorun = section->Get_bool("startcmd");
+								mainMenu.get_item("dos_win_autorun").check(winautorun).enable(true).refresh_item(mainMenu);
 							} else if (!strcasecmp(inputline.substr(0, 10).c_str(), "startwait=")) {
 								startwait = section->Get_bool("startwait");
 								mainMenu.get_item("dos_win_wait").check(startwait).enable(true).refresh_item(mainMenu);
